/*
 *               ParaStation
 *
 * Copyright (C) 1999-2004 ParTec AG, Karlsruhe
 * Copyright (C) 2005-2007 Cluster Competence Center GmbH, Munich
 *
 * $Id: pse.h 4639 2007-03-09 14:52:03Z eicker $
 *
 */
/**
 * @file
 * ParaStation Programming Environment
 *
 * $Id: pse.h 4639 2007-03-09 14:52:03Z eicker $
 *
 * @author
 * Norbert Eicker <eicker@par-tec.com>
 *
 */
#ifndef __PSE_H
#define __PSE_H

#include <stdint.h>
#include <sys/types.h>

#include "psnodes.h"

#ifdef __cplusplus
extern "C" {
#if 0
} /* <- just for emacs indentation */
#endif
#endif

/**
 * @brief Initialize PSE.
 *
 * Initialize PSE, the ParaStation Programming Environment. You have
 * to call this function before using any other function contained in
 * PSE. Otherwise the behavior of any other PSE function is
 * undetermined.
 *
 * @return No return value.
 * */
void PSE_initialize(void);

/**
 * @brief Create a partition.
 *
 * Create a partition of size @a num. This is mainly a wrapper around
 * @ref PSI_createPartition(), where the @a hwType used is set via
 * the @ref PSE_setHWType() / @ref PSE_setHWList() interface.
 *
 * Any process spawned by a process of the parallel task will reside
 * within the partition bound to this task. Therefore this function
 * has to be called befor any of @ref PSE_spawnMaster() or @ref
 * PSE_spawnTasks().
 *
 * @param num The number of nodes to be resevered for the partition.
 *
 * @return On success, the number of nodes in the partition is
 * returned or -1 if an error occurred.
 *
 * @see PSE_setHWType() PSE_setHWList() PSI_createPartition()
 * PSE_spawnMaster() PSE_spawnTasks()
 */
int PSE_getPartition(unsigned int num);

/**
 * @brief Get the size of the process group.
 *
 * Get the actual size of the process group.
 *
 * @return On success, the actual size of the process group is
 * returned, or -1, if an error occurred.
 */
int PSE_getSize(void);

/**
 * @brief Get the rank of the process.
 *
 * Get the rank of the actual process within the process group. The
 * rank of a process is positiv number smaller than the result of @ref
 * PSE_getSize() \b or \b -1. The rank is unique within the process
 * group.
 *
 * Different from MPI, in PSE the rank may be -1. It is assumed, that
 * the process with rank = -1 calls @ref PSE_spawnMaster() to spawn a
 * process with rank = 0 and become logger. The so spawned process
 * with rank = 0 is the master process in the sense of MPI and can
 * spawn further tasks using @ref PSE_spawnTasks(). Calls to @ref
 * PSE_spawnTasks() from within the process with rank = -1 will fail.
 *
 * The rank will never change during a process's lifetime.
 *
 * As long as processes are spawned via PSE_spawnTasks() which
 * acutally calls PSI_spawn(), the uniqueness of process ranks is
 * secured. If processes are spawned via PSI_spawnRank(), the user
 * itself is responsible for the uniqueness of process ranks within a
 * parallel task.
 *
 * @return On success, the actual rank of the process within the group
 * is returned, or -2, if an error occurred.
 *
 * @see PSE_getSize(), PSE_spawnMaster(), PSE_spawnTasks()
 * */
int PSE_getRank(void);

/**
 * @brief Deprecated form of PSE_initialize().
 *
 * Initializes PSE, the ParaStation Programming Environment. You have
 * to call this function before using any other function contained in
 * PSE. Otherwise the behavior of any other PSE function is
 * undetermined. You have to use this function instead of @ref
 * PSE_initialize() if you want to call the deprecated @ref
 * PSE_spawn().
 *
 * PSE is initialized and the aspired size of the process group is set
 * to @a NP. A sequent call of @ref PSE_spawn() with argument @a rank
 * = 0 will spawn @a NP-1 tasks.
 *
 * @param NP The aspired size of the process group.
 *
 * @param rank On return, @a rank contains the actual rank of the
 * process.
 *
 * @warning Deprecated form of @ref PSE_initialize(). Don't use this.
 *
 * @deprecated Better use @ref PSE_initialize() to initialize and @ref
 * PSE_getRank() to get the actual rank.
 *
 * @see PSE_initialize(), PSE_getRank(), PSE_spawn()
 * */
void PSE_init(int NP, int *rank);

/**
 * @brief Register to the parents task.
 *
 * Register the actual process to the parents task, so it's notified
 * thru a SIGTERM when the parent dies.
 *
 * If the registration to the parent fails, it is assumed that the
 * parent already has died. An error message is generated and the
 * process exits.
 *
 * @return No return value.
 * */
void PSE_registerToParent(void);

/**
 * @brief Set UID for spawns
 *
 * Set the UID for subsequently spawned processes to @a uid. This will
 * only affect processes spawned via PSI_spawnMaster() or
 * PSE_spawnTasks(). Furthermore the UID of the logger process which
 * will be created from within PSE_spawnMaster() will change to @a
 * uid. Only root (i.e. UID 0) is allowed to change the UID of spawned
 * processes.
 *
 * @param uid The UID of the processes to spawn.
 *
 * @return No return value.
 */
void PSE_setUID(uid_t uid);

/**
 * @brief Set hardware-type for PSE_getPartition().
 *
 * Set the hardware-type for @ref PSE_getPartition().
 *
 * If @ref PSE_getPartition() is called, the partition is constituted
 * from nodes which suppport all the hardware-types requested in @a
 * hwType. Subsequent calls to @ref PSE_spawnMaster() and @ref
 * PSE_spawnTasks() will only spawn processes within this
 * partition. For details on the spawning strategy look @ref
 * spawn_strategy "here".
 *
 * If no call to this function or to @ref PSE_setHWList() is made before
 * @ref PSE_getPartition() is called, the default hardware-type 0 is
 * used. This means, any node is accepted.
 *
 * @param hwType The hardware-type nodes have to support to get a
 * process spawned on. @a hwType is a bitwise or of the hardware-types
 * requested via 1<<INFO_request_hwindex() or 0. If @a hwType is 0, any
 * node is taken to spawn tasks on.
 *
 * @return No return value.
 *
 * @see PSE_getPartition() PSE_setHWList()
 * */
void PSE_setHWType(uint32_t hwType);


/**
 * @brief Set hardware-type for PSE_getPartition().
 *
 * Alternative form to set the hardware-type for @ref PSE_getPartition().
 *
 * If @ref PSE_getPartition() is called, the partition is constituted
 * from nodes which suppport all the hardware-types requested in @a
 * hwList.  Subsequent calls to @ref PSE_spawnMaster() and @ref
 * PSE_spawnTasks() will only spawn processes within this
 * partition. For details on the spawning strategy look @ref
 * spawn_strategy "here".
 *
 * If no call to this function or to @ref PSE_setHWType() is made
 * before @ref PSE_getPartition() is called, the default hardware-type
 * 0 is used. This means, any node is accepted.
 *
 * If one ore more of the hardware-types passed to this function are
 * unknown, the default hardware-type is set to the remaining ones
 * anyhow. The occurence of unknown hardware types is displayed by a
 * return value of -1.
 *
 * @param hwList A NULL terminated list of hardware names nodes have
 * to support to get a process spawned on. These will be resolved
 * using the parastation.conf configuration file, i.e. each hardware
 * name has to be defined there. Afterwards a @a hwType variable is
 * constructed and registered via @ref PSE_setHWType().
 *
 * @return If one or more hardware-types are unknown, -1 is
 * returned. Or 0, if all hardware-types are known. The default
 * hardware-type is set to the known ones in any case.
 *
 * @see PSE_getPartition() PSE_setHWType()
 */
int PSE_setHWList(char **hwList);

/**
 * @page spawn_strategy Spawning strategy
 *
 * To spawn the master process and subsequent tasks, a node pool is
 * constituted by @ref PSE_spawnMaster(). The nodes within the pool are
 * ordered depending on the requested ordering strategy and propagated
 * to spawned tasks. Any further spawning of tasks will leave the pool
 * and its ordering unchanged.
 *
 * Depending on its rank, a process will reside on a defined node
 * within the pool. Assume for example its rank is @a i, it will
 * reside on node number @a i of the pool. If more tasks are spawned
 * than the the pool contains nodes, the nodes are reused in a round
 * robin fashion.
 *
 * The spawning strategie can be influenced by using various
 * environment variables. These will steer on the one hand which nodes
 * will build the pool of nodes and on the other hand the ordering of
 * the nodes within this pool.
 *
 * Let's start with the ones that choose the nodes that form the
 * pool. There are three enviroment variable that control the pool,
 * PSI_NODES, PSI_HOSTS and PSI_HOSTFILE. The pool is build using the
 * following strategy
 *
 * - If PSI_NODES is present, use it to get the pool. PSI_NODES has to
 * contain a comma-separated list of node-number, i.e. positiv numbers
 * smaller than @a NrOfNodes from the parastation.conf configuration
 * file.
 *
 * - Otherwise if PSI_HOSTS is present, use this. PSI_HOSTS has to
 * contain a comma-separated list of hostnames. Each of them has to be
 * present in the parastation.conf configuration file.
 *
 * - If the pool is not build yet, use PSI_HOSTFILE. If PSI_HOSTFILE
 * is set, it has to contain a filename. The according file consists
 * of whitespace separated hostnames. Each of them has to be present
 * in the parastation.conf configuration file.
 *
 * - If none of the three addressed environment variables is present,
 * take all nodes managed by ParaStation to build the pool.
 *
 * To get into the pool, any node is tested if it is available and if
 * it supports the requested hardware-type set by @ref
 * PSE_setHWType().
 *
 * Be aware of the fact, that setting any of the environment variables
 * is kind of setting a static nodelist. This means, if any of the
 * nodes within the pool is not available, the utilisation of the
 * cluster may be suboptimal. On the other hand, this mechanism can be
 * used to implement a dynamical partitioning of the cluster. To
 * realize a more evolved distribution strategy, the initialization of
 * the environment variables may be done using an external batch
 * system like LSF, PBSpro or OpenPBS,
 *
 * When the pool is build, it may have to be sorted. The sorting is
 * steered using the environment variable PSI_NODES_SORT. Depending on
 * its value, one of the following sorting strategies is deployed to
 * the node pool:
 *
 * - PROC: Sort the pool depending on the number of processes managed
 * by ParaStation residing on the nodes. This is also the default if
 * PSI_NODES_SORT is not set.
 *
 * - LOAD or LOAD_1: Sort the pool depending on the load average
 * within the last minute on the nodes.
 *
 * - LOAD_5: Sort the pool depending on the load average within the
 * last 5 minutes on the nodes.
 *
 * - LOAD_15: Sort the pool depending on the load average within the
 * last 15 minutes on the nodes.
 *
 * - NONE or anything else: Don't sort the pool.
 * */

/**
 * @brief Spawn the master process.
 *
 * Spawns the master process and become logger. A process with rank =
 * 0 is spawned and the actual process will exec(2) to the
 * psilogger. It will serve the spawned process and all subsequently
 * spawned processes as an I/O daemon.
 *
 * Read more about spawning strategies @ref spawn_strategy "here".
 *
 * If an error occures, an error message is generated and the process
 * exits.
 *
 * @param argc The size of @a argv. This is usually equal to the argc
 * argument to main() of the actual process.
 *
 * @param argv The argument vector of the task to spawn. This is
 * usually equal to the argv argument to main() of the actual process.
 *
 * @return No return value. Actually, @a PSE_spawnMaster() never
 * returns.
 *
 * @warning Calls to this function from within processes where @ref
 * PSE_getRank() returns a value different from -1 will fail.
 *
 * @see PSE_getRank(), PSE_setHWType(), exec(2)
 * */
void PSE_spawnMaster(int argc, char *argv[]);


/**
 * @brief Spawn one or more tasks.
 *
 * Spawns @a num tasks. @a node and @a port are passed to the spawned
 * tasks. They can be determined there using @ref PSE_getMasterNode()
 * and @ref PSE_getMasterPort(). They may be used there to reconnect
 * to the spawning process.
 *
 * For spawning, the node pool constituted within @ref
 * PSE_spawnMaster() is used. Subsequent calls to @a PSE_spawnTasks()
 * will only spawn within this node pool. For details on the spawning
 * strategy look @ref spawn_strategy "here".
 *
 * If an error occures, an error message is generated and the process
 * exits.
 *
 * @param num The number of tasks to spawn.
 *
 * @param node The node number to pass to spawned processes. Usually
 * this is the node ID returned by the PSPort library using @ref
 * PSP_GetNodeID().
 *
 * @param port The port number to pass to spawned processes. Usually
 * this is the port number returned by the PSPort library using @ref
 * PSP_GetPortNo().
 *
 * @param argc The size of @a argv. This is usually equal to the argc
 * argument to main() of the actual process.
 *
 * @param argv The argument vector of the task to spawn. This is
 * usually equal to the argv argument to main() of the actual process.
 *
 * @return No return value.
 *
 * @warning Calls to this function from within processes where @ref
 * PSE_getRank() returns -1 will fail.
 *
 * @see PSE_spawnMaster(), PSE_getMasterNode(), PSE_getMasterPort(),
 * PSP_GetNodeID(), PSP_GetPortNo()
 * */
void PSE_spawnTasks(int num, int node, int port, int argc, char *argv[]);

/**
 * @brief Get the node ID of the master process.
 *
 * Get the ID of the node, where the master process resides. It is
 * passed from the spawning process as the @a node parameter to
 * PSE_spawnTasks() to spawned tasks. This is ususally the node ID
 * returned by the PSPort library using @ref PSP_GetNodeID().
 *
 * @return The node ID passed from the parent task as the @a node
 * parameter to @ref PSE_spawnTasks().
 *
 * @see PSE_spawnTasks(), PSP_GetNodeID()
 * */
int PSE_getMasterNode();

/**
 * @brief Get the port number of the master process.
 *
 * Get the port number of the master process. It is passed from the
 * spawning process as the @a port parameter to PSE_spawnTasks() to
 * spawned tasks. This is ususally the port number returned by the PSPort
 * library using @ref PSP_GetPortNo().
 *
 * @return The port number passed from the parent task as the @a port
 * parameter to @ref PSE_spawnTasks().
 *
 * @see PSE_spawnTasks(), PSP_GetPortNo()
 * */
int PSE_getMasterPort();

/**
 * @brief Spawn admin process.
 *
 * Spawn an admin process as describe in @a argc and @a argv to node
 * @a node. The rank of the spawned process is set as given in @a
 * rank. If the local rank is -1, the local process will exec(2) to
 * the psilogger. It will serve the spawned process (and all further
 * processes spawned by it and its descendents) as an I/O daemon.
 *
 * Spawning is done without allocating a partition. Only selected
 * users are allowed to spawn admin processes.
 *
 * If an error occures, an error message is generated.
 *
 * @param node ID of the node to spawn to.
 *
 * @param rank The rank of the spawned process. This is mainly used
 * within reconnection to the logger.
 *
 * @param argc The size of @a argv.
 *
 * @param argv The argument vector of the task to spawn.
 *
 * @param strictArgv Flag to prevent "smart" replacement of argv[0].
 *
 * @return If the calling process's rank is -1, this function never
 * returns. Either it becomes logger or it exits after creating an
 * error message. Otherwise upon success 0 is returned, or and errno
 * describing the error that occurred.
 *
 * @see PSE_getRank(), exec(2)
 * */
int PSE_spawnAdmin(PSnodes_ID_t node, unsigned int rank,
		   int argc, char *argv[], int strictArgv);

/**
 * @brief Deprecated form of PSE_spawnMaster() and PSE_spawnTasks()
 *
 * Spawns the master process if @a rank is -1, spawns further tasks if
 * @a rank is 0 and gets info about @a node and @a port if @a rank >
 * 0.
 *
 * If @a rank is 0 tasks are spawned until the total number of tasks
 * including the master process (i.e. @a rank = 0) but excluding the
 * logger process (i.e. @a rank = -1) is equal to the parameter @a NP of
 * @ref PSE_init().

 * After @a PSE_spawn() has returned, a task group consisting of @a NP
 * processes plus a logger process will exist. Furthermore all
 * processes have the same info about @a node and @a port.
 *
 * If an error occures, an error message is generated and the process
 * exits.
 *
 * You have to use the deprecated @ref PSE_init() instead of @ref
 * PSE_initialize() if you want to use this function.
 *
 * @param argc The size of @a argv. This is usually equal to the argc
 * argument to main() of the actual process.
 *
 * @param argv The argument vector of the task to spawn. This is
 * usually equal to the argv argument to main() of the actual process.
 *
 * @param node If @a rank = 0, the node number to pass to spawned
 * processes. Usually this is the node ID returned by the PSPort
 * library using @ref PSP_GetNodeID(). If @a rank > 0, on return the @a
 * node passed by the spawning process.
 *
 * @param port If @a rank = 0, the port number to pass to spawned
 * processes. Usually this is the port number returned by the PSPort
 * library using @ref PSP_GetPortNo(). If @a rank > 0, on return the @a
 * port passed by the spawning process.
 *
 * @param rank The rank of the calling process. This has to be the
 * result of @ref PSE_getRank().
 *
 * @return No return value.
 *
 * @warning Deprecated form of @ref PSE_spawnMaster() and @ref
 * PSE_spawnTasks(). Don't use this.
 *
 * @deprecated Better use @ref PSE_spawnMaster() to spawn the master
 * task, @ref PSE_spawnTasks() to spawn further task, @ref
 * PSE_registerToParent() to register to the parents task and @ref
 * PSE_getMasterNode() and @ref PSE_getMasterPort() to get the
 * required info.
 *
 * @see PSE_spawnMaster(), PSE_spawnTasks(), PSE_getMasterNode(),
 * PSE_getMasterPort(), PSE_init()
 * */
/* void PSE_spawn(int argc, char *argv[], int *node, int *port, int rank); */

/**
 * @brief Finish the actual process.
 *
 * Finish the actual process without shutting down all other processes
 * within the process group.
 *
 * @return No return value.
 *
 * @see PSE_abort()
 * */
void PSE_finalize(void);

/**
 * @brief Finish the actual process and shut down the whole process group.
 *
 * Finish the actual process and shut down all other processes within
 * the process group. @a code is returned to the calling process,
 * which is usually the forwarder.
 *
 * @param code The exit code
 *
 * @return No return value. PSE_abort() usually never returns, since
 * exit() is called.
 *
 * @see PSE_finalize(), exit(2)
 * */
void PSE_abort(int code);

/* For compatibility with old versions, to be removed soon */
/** Compatibility define @deprecated To be removed soon*/
#define PSEinit      PSE_init
/** Compatibility define @deprecated To be removed soon*/
#define PSEspawn     PSE_spawn
/** Compatibility define @deprecated To be removed soon*/
#define PSEfinalize  PSE_finalize
/** Compatibility define @deprecated To be removed soon*/
#define PSEabort     PSE_abort
/** Compatibility define @deprecated To be removed soon*/
#define PSEgetmyrank PSE_getRank
/** Compatibility define @deprecated To be removed soon*/
#define PSEgetsize   PSE_getSize

#ifdef __cplusplus
}/* extern "C" */
#endif

#endif /* __PSE_H */
